// shader type: gen 
// shader written by unknown on glslsandbox.com  
// shader translated to gles by Erogenous-Tones.com 
precision mediump float;
//
//
// .default fragment shader

varying vec2 tcoord;// 
uniform sampler2D tex;// texture one
uniform sampler2D tex2;// texture two
uniform vec2 tres;// size of texture (screen)
uniform vec4 fparams;// 4 floats coming in 
uniform ivec4 iparams;// 4 ints coming in
uniform float ftime;// 0.0 to 1.0
uniform int itime;// increases when ftime hits 1.0

//---------------------------------------------------------------------------------------------------
// metadata
#define PI 3.1415926535897932384626
float time = float(itime) + ftime;
vec2 resolution = tres;// for easier glslsandbox.com conversion

//f0:red shift:
//f1:outer size:
//f2:outer thick:
float f0 = mix(0.3, 0.95, fparams[0]);
float f1 = mix(0.05, 0.55, fparams[1]);
float f2 = mix(0.5, 0.9, fparams[2]);

#define TAU 6.28

vec3 repeat(vec3 pos, vec3 span) {
return mod(pos, span) - span / 2.0;
}

float vmax(vec2 v) {
return max(v.x, v.y);
}

void pR(inout vec2 p, float a) {
p = cos(a)*p + sin(a)*vec2(p.y, -p.x);
}


float sdf_smin(float a, float b, float k) {
    float h = clamp( 0.5+0.5*(b-a)/k, 0.0, 1.0 );
    return mix( b, a, h ) - k*h*(1.0-h);
}

float sdf_circ(vec2 pos, float rad) {
return length(pos)-rad;
}

float sdf_ring(vec2 pos, float rad) {
float thick = rad*f1;
return length( vec2( sdf_circ(pos, rad)) )-thick;
}

float sdf_box(vec2 pos, vec2 s) {
return vmax(abs(pos) - s);
}



//---------------------------------------------------------------------------------------------------
void main(void) {

vec2 pos = gl_FragCoord.xy / resolution - vec2(0.5);
pos.x *= resolution.x/resolution.y;

vec3 color = vec3(0.0);

float d = sdf_ring(pos, f2);


for(int i=0; i<6; i++)
{
float offset = float(i)/6.0*TAU;
float size = 0.1 + sin(offset*2.0+time*3.0)*.05;
float next = sdf_ring(pos+vec2(sin(time+offset)*.3, cos(time+offset)*.3), size);
d = sdf_smin(d, next, 0.10);
}



if(d < 0.0001)
{
color.r = 1.0;
color.g = 1.0+d*10.0;
color.b = 0.25 + f1;
}
else
{
color.r = 1.0-d*10. * f0;
color.b = fract(d*15.0)*.6 - d*0.5;
color.g = fract(d*5.0)*.4 - d*0.5;
}

gl_FragColor = vec4(color, 1.0);


} //endof main


