// shader type: gen 
// shader written by unknown on glslsandbox.com  
// shader translated to gles by Erogenous-Tones.com 
precision mediump float;
varying vec2 tcoord;// location
uniform sampler2D tex;// texture one
uniform sampler2D tex2;// texture two
uniform vec2 tres;// size of texture (screen)
uniform vec4 fparams;// 4 floats coming in
uniform ivec4 iparams;// 4 ints coming in
uniform float ftime;// 0.0 to 1.0
uniform int itime;// increases when ftime hits 1.0

//f0:rotate one:
//f1:rotate two:
//f2:pull zoom:
float f0 = mix(0.05, 0.95, fparams[0]);
float f1 = mix(0.05, 0.95, fparams[1]);
float f2 = mix(0.05, 0.95, fparams[2]);

float time = float(itime) + ftime;
vec2 resolution = tres;
// from http://glslsandbox.com/e#47905.0

#define detail 1
#define steps 2

#define rot(spin) mat2(cos(spin),sin(spin),-sin(spin),cos(spin))

//random function from https://www.shadertoy.com/view/MlsXDf
//float rnd(vec4 v) { return fract(4e4*sin(dot(v,vec4(13.46,41.74,-73.36,14.24))+17.34)); }
float rnd(vec4 v){ return fract(sin(v.x)*43758.5453); }

//0 is empty, 1 is subdivide and 2 is full
int getvoxel(vec3 p, float size) {
    if (p.x == 0.0 && p.y == 0.0) {
        return 0;
    }
    
    float val = rnd(vec4(p,size));
    
    /*if (val < 0.5) {
        return 0;
    } else if (val < 0.8) {
        return 1;
    } else {
        return 2;
    }*/
    
    return int(val*val*3.0);
}

//ray-cube intersection, on the inside of the cube
vec3 voxel(vec3 ro, vec3 rd, float size) {
    size *= 0.5;
    vec3 hit = -(sign(rd)*(ro-size)-size)/max(abs(rd),0.001);
    return hit;
}

void main() {
    gl_FragColor = vec4(0.0);
    vec2 uv = (gl_FragCoord.xy * 2.0 - resolution.xy) /resolution.y / vec2(f2);
    float size = 1.0;
    
    vec3 ro = vec3(0.5+sin(time)*0.4,0.5+cos(time)*0.4,time);
    vec3 rd = normalize(vec3(uv,1.0));
    
    rd.xz *= rot(f0/resolution.x*3.14-3.14*f0);
    rd.yz *= rot(f1/resolution.y*3.14-3.14*f1);
    
    vec3 lro = mod(ro,size);
    vec3 fro = ro-lro;
    vec3 mask;
    bool exitoct = false;
    int recursions = 0;
    float dist = 0.0;
    int i = 0;
    float edge = 1.0;
    
    //the octree traverser loop
    //each iteration i either:
    // - check if i need to go up a level
    // - check if i need to go down a level
    // - check if i hit a cube
    // - go one step forward if cube is empty
    for (int i = 0; i < steps; i++) {
        int voxelstate = getvoxel(fro,size);
        
        //i go up a level
        if (exitoct) {
            vec3 newfro = floor(fro/size*0.5)*size*2.0;
            
            lro += fro-newfro;
            fro = newfro;
            
            recursions--;
            size *= 2.0;
            
            exitoct = (recursions > 0) && (abs(dot(mod(fro/size+0.5,2.0)-1.0+mask*sign(rd)*0.5,mask))<0.1);
        }
        //subdivide
        else if(voxelstate == 1) {
            //if i have rached the octree limit, break
            if(recursions>detail) break;
            
            recursions++;
            size *= 0.5;
            
            //find which of the 8 voxels i will enter
            vec3 mask2 = step(vec3(size),lro);
            fro += mask2*size;
            lro -= mask2*size;
        }
        //move forward
        else if (voxelstate == 0) {
            //raycast and find distance to nearest voxel surface in ray direction
            vec3 hit = voxel(lro, rd, size);
            if (hit.x < min(hit.y,hit.z)) {
                mask = vec3(1,0,0);
            } else if (hit.y < hit.z) {
                mask = vec3(0,1,0);
            } else {
                mask = vec3(0,0,1);
            }
            float len = dot(hit,mask);
            
            //moving forward in ray direction, and checking if i need to go up a level
            ro += rd*len;
            lro += rd*len-mask*sign(rd)*size;
            vec3 newfro = fro+mask*sign(rd)*size;
            exitoct = (floor(newfro/size*0.5+0.25)!=floor(fro/size*0.5+0.25))&&(recursions>0);
            fro = newfro;
        }
        else {
            break;
        }
    }
    
    if(i < steps) {
    float val = fract(dot(fro,vec3(15.23,754.345,3.454)));
        vec3 normal = mask*sign(rd);
        vec3 color = sin(val*vec3(39.896,57.3225,48.25))*0.5+0.5;
    gl_FragColor = vec4(color*(normal*0.25+0.75),1.0);
        
        vec3 q = abs(lro/size-0.5)*(1.0-mask);
        edge = clamp(-(max(max(q.x,q.y),q.z)-0.5)*20.0*size,0.0,edge);
        gl_FragColor *= vec4(edge);
    }
}


