// shader type: gen 
// shader written by unknown on glslsandbox.com  
// shader translated to gles by Erogenous-Tones.com 
precision mediump float;
varying vec2 tcoord;// 
uniform sampler2D tex;// texture one
uniform sampler2D tex2;// texture two
uniform vec2 tres;// size of texture (screen)
uniform vec4 fparams;// 4 floats coming in 
uniform ivec4 iparams;// 4 ints coming in
uniform float ftime;// 0.0 to 1.0
uniform int itime;// increases when ftime hits 1.0
//f0:x view:
//f1:y view:
//f2:diag zoom:

float f0 = mix(0.05, 0.95, fparams[0]);
float f1 = mix(0.05, 0.95, fparams[1]);
float f2 = mix(0.05, 0.95, fparams[2]);


// http://glslsandbox.com/e#39884.0

float time = float(itime) + ftime;


#define PI 3.14159265358979323846
#define TWO_PI 6.28318530717958647693

vec2 rotate2D (vec2 _st, float _angle) {
  _st -= 0.5;
  _st =  mat2(cos(_angle),-sin(_angle),
              sin(_angle),cos(_angle)) * _st;
  _st += 0.5;
  return _st;
}

vec2 tile (vec2 _st, float _zoom) {
  _st *= _zoom;
  return fract(_st);
}

vec2 rotateTile(vec2 _st){
    _st *= 2.0;

    float index = 0.0;    
    if (fract(_st.x * f0) > 0.5){
        index += 1.0;
    }
    if (fract(_st.y * f1) > 0.5){
        index += 2.0;
    }

    _st = fract(_st);

    if(index == 1.0){
        _st = rotate2D(_st,PI*0.5);
    } else if(index == 2.0){
        _st = rotate2D(_st,PI*-0.5);
    } else if(index == 3.0){
        _st = rotate2D(_st,PI);
    }

    return _st;
}

// Based on https://www.shadertoy.com/view/4sSSzG
float triangle (vec2 _st, 
                vec2 _p0, vec2 _p1, vec2 _p2, 
                float _smoothness) {
  vec3 e0, e1, e2;

  e0.xy = normalize(_p1 - _p0).yx * vec2(+1.0, -1.0);
  e1.xy = normalize(_p2 - _p1).yx * vec2(+1.0, -1.0);
  e2.xy = normalize(_p0 - _p2).yx * vec2(+1.0, -1.0);

  e0.z = dot(e0.xy, _p0) - _smoothness;
  e1.z = dot(e1.xy, _p1) - _smoothness;
  e2.z = dot(e2.xy, _p2) - _smoothness;

  float a = max(0.0, dot(e0.xy, _st) - e0.z);
  float b = max(0.0, dot(e1.xy, _st) - e1.z);
  float c = max(0.0, dot(e2.xy, _st) - e2.z);

  return smoothstep(_smoothness * 2.0, 
                    1e-7, 
                    length(vec3(a, b, c)));
}

//---------------------------------------------------------------------------------------------------
void main(void) {

    vec2 st = gl_FragCoord.xy/tres.xy;

    st = tile(st,6.0 * f2);
    st = rotateTile(st);
    
    float pattern = 0.0;

    st = rotate2D(st,-PI*time*0.25);
    pattern =   triangle(st, 
                         vec2(0.30,-0.5), 
                         vec2(0.70,0.-0.5), 
                         vec2(0.5,1.0), 
                         0.01);

    vec3 color = vec3(pattern);

    gl_FragColor = vec4(color,1.0); 
   
} //endof main


