// shader type: gen 
// shader written by unknown on glslsandbox.com  
// shader translated to gles by Erogenous-Tones.com 
precision mediump float;
varying vec2 tcoord;// 
uniform sampler2D tex;// texture one
uniform sampler2D tex2;// texture two
uniform vec2 tres;// size of texture (screen)
uniform vec4 fparams;// 4 floats coming in 
uniform ivec4 iparams;// 4 ints coming in
uniform float ftime;// 0.0 to 1.0
uniform int itime;// increases when ftime hits 1.0
//f0:color offset:
//f1:color agitate:
//f2:color cycle:
float f0 = mix(0.05, 0.95, fparams[0]);
float f1 = mix(0.05, 0.95, fparams[1]);
float f2 = mix(0.05, 0.95, fparams[2]);

// https://www.shadertoy.com/view/Mssfz4
float time = float(itime) + ftime;

float noise(in vec2 position) {
    return fract(sin(dot(position.xy, vec2(12.9898,78.233)))* 43758.5453123);
}

// Value Noise courtesy of Book of Shaders // https://thebookofshaders.com/11/
float noise2d(vec2 uv) {
    
    vec2 pos = floor(uv);
    vec2 fractional = fract(uv);
    
    // four corners
    float a = noise(pos);// bottom left
    float b = noise(pos + vec2(1., 0.));// bottom right
    float c = noise(pos + vec2(1., 1.));// top right
    float d = noise(pos + vec2(0., 1.));// top left
    
    vec2 intermix = smoothstep(0., 1., fractional);
    
    float value = mix(a, b, intermix.x);
    value += (d - a) * intermix.y * (1.0 - intermix.x) + f2;
    value += (c - b) * intermix.x * intermix.y + f2;
    
    return value;
}

// Fractal noise courtesy of iq // https://www.shadertoy.com/view/XdXGW8
float fractalNoise2d(vec2 uv) {
    uv *= 3.0;
    uv.x -= time / 5.0;
    uv.y += sin(time / 5.0) * 2.0;
    
    mat2 rotate = mat2(1.6 - sin(time / 100.0) / 10.0, 1.2, -1.2, 2.6 * f1 * 0.05);
    
    float value = 0.5 * noise2d(uv);
    uv *= rotate;
    value += 0.25 * noise2d(uv);
    uv *= rotate;
    value += 0.125 * noise2d(uv);
    uv *= rotate;
    value += 0.0625 * noise2d(uv);
    
    return value;
}

vec2 rotate2D(vec2 _st, float _angle){
    _st -= 0.5;
    _st =  mat2(cos(_angle),-sin(_angle),
                sin(_angle),cos(_angle)) * _st;
    _st += 0.5;
    return _st;
}

//---------------------------------------------------------------------------------------------------
void main(void) {
    vec2 uv = tcoord.xy * 0.6;
    
    float noise = fractalNoise2d(uv);
    uv = rotate2D(uv, noise*120.) ;
    
    uv = (f0 * 4.0 +1.0) * uv;
    float plasma = noise*(sin(uv.x)+cos(uv.y))*5.;
    
    vec3 col = vec3(sin((-plasma)+time),
                    cos((-plasma+.4)+time),
                    sin((-plasma+.5)+time + f0));

gl_FragColor = vec4(col,1.0);


} //endof main


