// shader type: gen 
// shader written by unknown on glslsandbox.com  
// shader translated to gles by Erogenous-Tones.com 
precision mediump float;
varying vec2 tcoord;// 
uniform sampler2D tex;// texture one
uniform sampler2D tex2;// texture two
uniform vec2 tres;// size of texture (screen)
uniform vec4 fparams;// 4 floats coming in 
uniform ivec4 iparams;// 4 ints coming in
uniform float ftime;// 0.0 to 1.0
uniform int itime;// increases when ftime hits 1.0
//f0:complexity:
//f1:density:
//f2:shading:

float f0 = mix(0.05, 0.95, fparams[0]);
float f1 = mix(0.05, 0.95, fparams[1]);
float f2 = mix(0.05, 0.95, fparams[2]);

//https://www.shadertoy.com/view/Md33zB

float time = float(itime) + ftime;
vec2 resolution = tres;// for easier glslsandbox.com conversion
//---------------------------------------------------------------------------------------------------

const vec3 inkColor = vec3(0.01, 0.01, 0.1);
const vec3 paperColor = vec3(1.0, 0.98, 0.94);

float speed =  1.5; 
float shadeContrast = 0.2 / f2;

//3D simplex noise from: https://www.shadertoy.com/view/XsX3zB
const float F3 =  0.3333333;
const float G3 =  0.1666667;

vec3 random3(vec3 c) {
    float j = 4096.0*sin(dot(c,vec3(17.0, 59.4, 15.0)));
    vec3 r;
    r.z = fract(512.0*j);
    j *= .125;
    r.x = fract(512.0*j);
    j *= .125;
    r.y = fract(512.0*j);
    return r-0.5;
}

float simplex3d(vec3 p) {
 vec3 s = floor(p + dot(p, vec3(F3)));
 vec3 x = p - s + dot(s, vec3(G3));
 
 vec3 e = step(vec3(0.0), x - x.yzx);
 vec3 i1 = e*(1.0 - e.zxy);
 vec3 i2 = 1.0 - e.zxy*(1.0 - e);
 
 vec3 x1 = x - i1 + G3;
 vec3 x2 = x - i2 + 2.0*G3;
 vec3 x3 = x - 1.0 + 3.0*G3;
 
 vec4 w, d;
 
 w.x = dot(x, x);
 w.y = dot(x1, x1);
 w.z = dot(x2, x2);
 w.w = dot(x3, x3);
 
 w = max(0.6 - w, 0.0);
 
 d.x = dot(random3(s), x);
 d.y = dot(random3(s + i1), x1);
 d.z = dot(random3(s + i2), x2);
 d.w = dot(random3(s + 1.0), x3);
 
 w *= w;
 w *= w;
 d *= w;
 
 return dot(d, vec4(52.0));
}

float fbm(vec3 p) {
float f = 0.0;
float frequency = 1.0 + f0;
float amplitude = 0.5 / f1  +0.2;
for (int i = 0; i < 2; i++) {
f += simplex3d(p * frequency) * amplitude;
amplitude *= 0.5;
frequency *= 2.0 + float(i) / 100.0;
}
return min(f, 1.0);
}


void main(void) {

    //Setup coordinates
    vec2 uv = 1.0 - gl_FragCoord.xy / tres.xy;
    vec2 coord = 1.0 - uv * 2.0;
    uv.x = 1.0 - abs(1.0 - uv.x * 2.0);
    vec3 p = vec3(uv, time * speed);
    
    //Sample a noise function
    float blot = fbm(p * 3.0 + 8.0);
    float shade = fbm(p * 2.0 + 16.0);
    
    //Threshold
    blot = (blot + (sqrt(uv.x) - abs(0.5 - uv.y)));
    blot = smoothstep(0.65, 0.71, blot) * max(1.0 - shade * shadeContrast, 0.0);
    
    //Color
    gl_FragColor = vec4(mix(paperColor, inkColor, blot), 1.0);
    gl_FragColor.rgb *= 1.0 - pow(max(length(coord) - 0.5, 0.0), 5.0);
   
} //endof main


