// shader type: gen 
// shader written by unknown on glslsandbox.com  
// shader translated to gles by Erogenous-Tones.com 
precision mediump float;
varying vec2 tcoord;// 
uniform sampler2D tex;// texture one
uniform sampler2D tex2;// texture two
uniform vec2 tres;// size of texture (screen)
uniform vec4 fparams;// 4 floats coming in 
uniform ivec4 iparams;// 4 ints coming in
uniform float ftime;// 0.0 to 1.0
uniform int itime;// increases when ftime hits 1.0
//f0:mod 1:
//f1:mod 2:
//f2:mod 3:

float f0 = mix(0.05, 0.95, fparams[0]);
float f1 = mix(0.3, 0.95, fparams[1]);
float f2 = mix(0.1, 0.95, fparams[2]);


float time = float(itime) + ftime * 2.;

#define STEPS 6
#define STEP_SIZE 0.3
#define MIN_DISTANCE 0.01

mat4 inverse(mat4 m) {
  float
      a00 = m[0][0], a01 = m[0][1], a02 = m[0][2], a03 = m[0][3],
      a10 = m[1][0], a11 = m[1][1], a12 = m[1][2], a13 = m[1][3],
      a20 = m[2][0], a21 = m[2][1], a22 = m[2][2], a23 = m[2][3],
      a30 = m[3][0], a31 = m[3][1], a32 = m[3][2], a33 = m[3][3],

      b00 = a00 * a11 - a01 * a10,
      b01 = a00 * a12 - a02 * a10,
      b02 = a00 * a13 - a03 * a10,
      b03 = a01 * a12 - a02 * a11,
      b04 = a01 * a13 - a03 * a11,
      b05 = a02 * a13 - a03 * a12,
      b06 = a20 * a31 - a21 * a30,
      b07 = a20 * a32 - a22 * a30,
      b08 = a20 * a33 - a23 * a30,
      b09 = a21 * a32 - a22 * a31,
      b10 = a21 * a33 - a23 * a31,
      b11 = a22 * a33 - a23 * a32,

      det = b00 * b11 - b01 * b10 + b02 * b09 + b03 * b08 - b04 * b07 + b05 * b06;

  return mat4(
      a11 * b11 - a12 * b10 + a13 * b09,
      a02 * b10 - a01 * b11 - a03 * b09,
      a31 * b05 - a32 * b04 + a33 * b03,
      a22 * b04 - a21 * b05 - a23 * b03,
      a12 * b08 - a10 * b11 - a13 * b07,
      a00 * b11 - a02 * b08 + a03 * b07,
      a32 * b02 - a30 * b05 - a33 * b01,
      a20 * b05 - a22 * b02 + a23 * b01,
      a10 * b10 - a11 * b08 + a13 * b06,
      a01 * b08 - a00 * b10 - a03 * b06,
      a30 * b04 - a31 * b02 + a33 * b00,
      a21 * b02 - a20 * b04 - a23 * b00,
      a11 * b07 - a10 * b09 - a12 * b06,
      a00 * b09 - a01 * b07 + a02 * b06,
      a31 * b01 - a30 * b03 - a32 * b00,
      a20 * b03 - a21 * b01 + a22 * b00) / det;
}

vec2 opU( vec2 d1, vec2 d2 )
{
    return (d1.x < d2.x) ? d1 : d2;
}

float vmax(vec3 v)
{
return max(max(v.x, v.y), v.z);
}

float box(vec3 p, vec3 c, vec3 s)
{
return vmax(abs(p-c) - s);
}


float sphere(vec3 p, vec3 center, float radius)
{
return distance(p,center) - radius;
}

mat4 constructMatrix() {

//rotate by a certain angle around z axis

float a = time/1.;

mat4 rotMat = mat4( vec4(cos(a),-sin(a),0,0),
    vec4(sin(a),cos(a),0,0),
    vec4(0,0,1,0),
    vec4(0,0,0,1));

//take the inverse of rotMat
return inverse(rotMat);
}

//this is a signed distance function now
float map (vec3 p)
{
//multiply p by the model matrix
vec4 p2 = vec4(p,0.);
 vec4 p3 = constructMatrix() * p2;
 p = vec3(p3.x,p3.y,p3.z);


float x = min(
sphere(p,vec3(0.0,0.0,1.), 1.),
sphere(p,vec3(.7,.9,1.3), 1.)
);

x = min(x,
sphere(p,vec3(6.0,6.0,1.),.2)
);

x = min(x,
sphere(p,vec3(-6.0,6.0,0.),.8)
);


x = min(x,
sphere(p,vec3(-4.0,6.0,0.),.8)
);

x = min(x,
sphere(p,vec3(-4.0,-6.0,6.),1.8)
);
x = min(x,
sphere(p,vec3(6.0,-6.0,-.6),2.)
);

x = min(x,
box(p,vec3(6.0,1.0,-.6),vec3(1.))
);

return x;
}



vec3 normal (vec3 p)
{
const float eps = 0.01;
 
return normalize
(vec3
(map(p + vec3(eps, 0, 0)) - map(p - vec3(eps, 0, 0)),
map(p + vec3(0, eps, 0)) - map(p - vec3(0, eps, 0)),
map(p + vec3(0, 0, eps)) - map(p - vec3(0, 0, eps))
)
);
}
float renderSurfaceBP(vec3 p, vec3 v)
{
vec3 n = normal(p);
vec3 l = vec3(0.,0.,1.);

vec3 h = (l-v)/length(l-v);

float spec = 1.;

return pow(dot(n,h),spec)*2.;

}

float renderSurfaceL(vec3 p)
{
vec3 n = normal(p);
vec3 l = vec3(.5,0.,1.);
return dot(n,l);

}

float sierpinski(vec3 pos) {
vec3 a1 = vec3(1,1,1);
vec3 a2 = vec3(-1,-1,1);
vec3 a3 = vec3(1,-1,-1);
vec3 a4 = vec3(-1,1,-1);
vec3 c;

float dist, d;
int n2 = 0;
for (int n = 0; n < 2; n++) {
c = a1 * f0; 
dist = length(pos-a1);
        d = length(pos-a2); if (d < dist) { c = a2; dist=d; }
 d = length(pos-a3); if (d < dist) { c = a3; dist=d; }
 d = length(pos-a4); if (d < dist) { c = a4; dist=d; }
pos = 2.*pos-c*(2.-1.0) /f1;
n2 = n;
}

float finDist = length(pos) * pow(2., float(-n2));
return renderSurfaceL(pos);
}

float trace(vec3 pos, vec3 viewDirection) {
for (int i = 1; i <= STEPS; i++) {
float distance = map(pos);
if (distance < MIN_DISTANCE) {
return renderSurfaceL(pos);
}

pos += distance * viewDirection;
}
return 1.;
}


//---------------------------------------------------------------------------------------------------
// http://glslsandbox.com/e#41805.0
void main(void) {


vec2 position = ( gl_FragCoord.xy / tres.xy ) * 2.0 - 1.0;
position = position * tres.xy / max(tres.x, tres.y);

float color = 1.0;

//vec3 from = vec3(-f0*5.-2.,-f1*5.-2.,-10.0);
vec3 p = vec3(position*f2 * 4.,1.0);

//vec3 viewDirection = p;


//color = trace(from,viewDirection); 
float color2 = sierpinski(p);

gl_FragColor = vec4(vec3(color2),1.);// vec3( color, color * 0.5, sin( color + time  ) * 0.75 ), 1.0 );
   
} //endof main


