// shader type: gen 
// shader written by unknown on glslsandbox.com  
// shader translated to gles by Erogenous-Tones.com 
precision mediump float;
varying vec2 tcoord;//
uniform sampler2D tex;// texture one
uniform sampler2D tex2;// texture two
uniform vec2 tres;// size of texture (screen)
uniform vec4 fparams;// 4 floats coming in
uniform ivec4 iparams;// 4 ints coming in
uniform float ftime;// 0.0 to 1.0
uniform int itime;// increases when ftime hits 1.0
//f0:x pos:
//f1:y pos:
//f2:zoom:

float f0 = mix(0.05, 0.95, fparams[0]);
float f1 = mix(0.05, 0.95, fparams[1]);
float f2 = mix(0.05, 0.95, fparams[2]);

float time = float(itime) + ftime;
vec2 resolution = tres;
//---------------------------------------------------------------------------------------------------

vec3 firePalette(float i){

    float T = 1400. + 1300.*i; // Temperature range (in Kelvin).
    vec3 L = vec3(7.4, 5.6, 4.4); // Red, green, blue wavelengths (in hundreds of nanometers).
    L = pow(L,vec3(5.0)) * (exp(1.43876719683e5/(T*L))-1.0);
    return 1.0-exp(-5e8/L); // Exposure level. Set to "50." For "70," change the "5" to a "7," etc.
}

// Hash function. This particular one probably doesn't disperse things quite as nicely as some 
// of the others around, but it's compact, and seems to work.
//
vec3 hash33(vec3 p){ 
    
    float n = sin(dot(p, vec3(7, 157, 113)));    
    return fract(vec3(2097152, 262144, 32768)*n); 
}

float voronoi(vec3 p){

vec3 b, r, g = floor(p);
p = fract(p); // "p -= g;" works on some GPUs, but not all, for some annoying reason.

float d = 1.0; 
     
    
    b = vec3(0, 0, -1);
    r = b - p + hash33(g+b);
    d = min(d, dot(r,r));
    
    b.z = 0.0;
    r = b - p + hash33(g+b);
    d = min(d, dot(r,r));
    
    b.z = 1.;
    r = b - p + hash33(g+b);
    d = min(d, dot(r,r));
    

return d; // Range: [0, 1]
}

float noiseLayers(in vec3 p) {

    vec3 t = vec3(0., 0., p.z+time*1.5);

    const int iter = 2; // Just five layers is enough.
    float tot = 0., sum = 0., amp = 1.; // Total, sum, amplitude.

    for (int i = 0; i < iter; i++) {
        tot += voronoi(p + t) * amp; // Add the layer to the total.
        p *= 2.0; // Position multiplied by two.
        t *= 1.5; // Time multiplied by less than two.
        sum += amp; // Sum of amplitudes.
        amp *= 0.5; // Decrease successive layer amplitude, as normal.
    }
    
    return tot/sum; // Range: [0, 1].
}

//---------------------------------------------------------------------------------------------------
// source : http://glslsandbox.com/e#35359.4 // Original code: https://www.shadertoy.com/view/4tlSzl
void main(){
    // Screen coordinates.
vec2 uv = (gl_FragCoord.xy - tres.xy*0.5) / tres.y;

// Shifting the central position around, just a little, to simulate a 
// moving camera, albeit a pretty lame one.
//uv += vec2(sin(time*0.5)*0.25, cos(time*0.5)*0.125);

uv += 1.0 - (2. * vec2(f0,f1));


    // Constructing the unit ray. 
vec3 rd = normalize(vec3(uv.x, uv.y, 3.1415926535898/8.)) * (f2 * 2. + .3);

    // Rotating the ray about the XY plane, to simulate a rolling camera.
float cs = cos(time*0.25), si = sin(time*0.25);
    // Apparently "r *= rM" can break in some older browsers.
rd.xy = rd.xy*mat2(cs, -si, si, cs); 

// Passing a unit ray multiple into the Voronoi layer function, which 
// is nothing more than an fBm setup with some time dialation.
float c = noiseLayers(rd*2.);

// Optional: Adding a bit of random noise for a subtle dust effect. 
c = max(c + dot(hash33(rd)*2.-1., vec3(0.015)), 0.);

    // Coloring:
    
    // Nebula.
    c *= sqrt(c)*1.5; // Contrast.
    vec3 col = firePalette(c); // Palettization.
    col = mix(col, col.zyx*0.1+c*0.9, (1.+rd.x+rd.y)*0.45 ); // Color dispersion.
    
    // The fire palette on its own. Perhaps a little too much fire color.
    // c = pow(c, 1.33)*1.33;
    // vec3 col =  firePalette(c);
   
    // Black and white, just to keep the art students happy. :)
// c *= sqrt(c)*1.5;
// vec3 col = vec3(c);

// Done.
gl_FragColor = vec4(clamp(col, 0., 1.), 1.);
}





